#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include "task.h"

// iDevice Breathing LED
// https://github.com/adafruit/iCufflinks/
uint8_t Fader_breathing[] PROGMEM = {
	1, 1, 2, 3, 5, 8, 11, 15, 20, 25, 30, 36, 43, 49, 56, 64, 72, 80, 88, 97,
	105, 114, 123, 132, 141, 150, 158, 167, 175, 183, 191, 199, 206, 212, 219,
	225, 230, 235, 240, 244, 247, 250, 252, 253, 254, 255, 254, 253, 252, 250,
	247, 244, 240, 235, 230, 225, 219, 212, 206, 199, 191, 183, 175, 167, 158,
	150, 141, 132, 123, 114, 105, 97, 88, 80, 72, 64, 56, 49, 43, 36, 30, 25,
	20, 15, 11, 8, 5, 3, 2, 1, 0
};

// Led_run
// Run function for Led task.
static void Led_run(void *_self, uint32_t now) {
	static uint8_t i;
	uint8_t val;
	if (!( val = pgm_read_byte(&Fader_breathing[i]) )) {
		i = 0;
		val = pgm_read_byte(&Fader_breathing[i]);
	}
	i++;
	OCR0A = val;
	Task_incRunTime(_self, 40);
}

// Led
// Led task constructor.
Task_t* Led(void) {
	static Task_t led;
	led.canRun = TimedTask_canRun;
	led.run = Led_run;
	led.runTime = 0;
	return &led;
}

// Timer1 interrupt
ISR( TIMER1_COMPA_vect ) {
	tick();
}

int main(void) {
	// IO setup
	DDRD = (1<<PD6) | (1<<PD7);
	PORTD = (1<<PD3);
	
	// Timer0
	// non-inverted PWM output, Fast PWM (Mode 3)
	TCCR0A = (1<<COM0A1) | (1<<WGM01) | (1<<WGM00);
	// Fast PWM (Mode 3), CLK/256
	TCCR0B = (1<<CS02);
	OCR0A = 0;
	
	// Timer1
	// every 1ms
	OCR1A = 12000;
	// OCR Interrupt
	TIMSK1 = (1<<OCIE1A);
	// CLK/1, CTC Mode
	TCCR1B = (1<<WGM12) | (1<<CS10);
	
	sei();
	
	// Initialize tasks
	// The Led task is an example of a simple timed task.
	// Implementing a canRun-function for it is not necessary.
	Task_t *led = Led();
	// 
	Task_t *tasks[] = {led};
	
	// Call TaskScheduler and we're done.
	TaskScheduler(tasks, sizeof(tasks)/sizeof(Task_t*));
	
	return 0;
}